# Generated by Django 3.2.13 on 2022-10-06 14:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0012_servicerevision_is_default"),
    ]

    operations = [
        migrations.AlterField(
            model_name="serviceusageevent",
            name="question",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="service_usage_events",
                to="django_twined.question",
            ),
        ),
    ]
