# Disable for migrations:
# pylint: disable=missing-docstring

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0004_add_timestamps"),
    ]

    operations = [
        migrations.AlterField(
            model_name="question",
            name="answered",
            field=models.DateTimeField(
                blank=True, editable=False, help_text="When the question was answered", null=True
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="asked",
            field=models.DateTimeField(blank=True, editable=False, help_text="When the question was asked", null=True),
        ),
    ]
