question_basic_fieldset = (
    None,
    {
        "fields": (
            "id",
            "service_revision",
            "asked",
            "answered",
        )
    },
)

question_db_input_values_fieldset = ("Input Values", {"classes": ("collapse",), "fields": ("input_values",)})
question_db_output_values_fieldset = ("Output Values", {"classes": ("collapse",), "fields": ("output_values",)})
