# Generated by Django 3.2.13 on 2022-06-06 13:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0010_alter_servicerevision_project_name"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="servicerevision",
            name="topic_id",
        ),
    ]
