# Generated by Django 3.2.13 on 2022-10-04 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0011_remove_servicerevision_topic_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="servicerevision",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="Set true to use this tagged revision as the default version for this name and namespace",
            ),
        ),
    ]
