{% extends 'admin/change_form.html' %}
{% load question_ask_row %}

{% block object-tools-items %}
    {% if show_duplicate %}
        <li><a href="?duplicate=True">Duplicate</a></li>
    {% endif %}
    {{ block.super }}
{% endblock %}

<!-- Add a save_and_ask button -->
{% block submit_buttons_bottom %}{% question_ask_row %}{% endblock %}
